"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../Styles/theme");
const Badge = props => {
    const { text, size = 'small', corner = false, dot = false, overflowCount = 99 } = props;
    const isNumber = !Number.isNaN(text);
    let showText = text;
    if (isNumber) {
        showText = +text > overflowCount ? `${overflowCount}+` : text;
    }
    const cls = [styles.badge];
    if (size === 'large') {
        cls.push(styles.large);
    }
    else {
        cls.push(styles.small);
    }
    if (dot) {
        cls.push(styles.dot);
    }
    if (corner) {
        cls.push(styles.corner);
        if (dot) {
            cls.push(styles.dotCorner);
        }
        else if (size === 'large') {
            cls.push(styles.largeCorner);
        }
        else {
            cls.push(styles.smallCorner);
        }
    }
    if (!showText && !corner) {
        return null;
    }
    return (<react_native_1.View style={cls}>
      {!dot && <react_native_1.Text style={[styles.text, size === 'large' ? styles.largeText : styles.smallText]}>{showText}</react_native_1.Text>}
    </react_native_1.View>);
};
const styles = react_native_1.StyleSheet.create({
    badge: {
        width: 'auto',
        height: 'auto',
        display: 'flex',
        flexDirection: 'row',
        alignItems: 'center',
        backgroundColor: theme_1.Theme.fillRed,
    },
    corner: {
        position: 'absolute',
    },
    text: {
        color: theme_1.Theme.colorTextBaseInverse,
        textAlignVertical: 'top',
    },
    large: {
        borderRadius: 10,
        paddingHorizontal: 7,
    },
    largeCorner: {
        top: -8,
        right: -18,
    },
    largeText: {
        fontSize: 12,
    },
    small: {
        borderRadius: 12,
        paddingHorizontal: 5,
    },
    smallCorner: {
        top: -6,
        right: -12,
    },
    smallText: {
        fontSize: 10,
    },
    dot: {
        width: 6,
        height: 6,
        borderRadius: 3,
        paddingHorizontal: 0,
    },
    dotCorner: {
        top: -3,
        right: -3,
    },
});
exports.default = Badge;
