"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const xmldom_1 = __importDefault(require("xmldom"));
const react_native_svg_1 = __importStar(require("react-native-svg"));
const utils_1 = require("./utils");
const { resolveAssetSource } = react_native_1.Image;
const ACCEPTED_SVG_ELEMENTS = [
    'svg',
    'g',
    'circle',
    'path',
    'rect',
    'linearGradient',
    'radialGradient',
    'stop',
    'ellipse',
    'polygon',
];
// Attributes from SVG elements that are mapped directly.
const SVG_ATTS = ['viewBox'];
const G_ATTS = ['id'];
const CIRCLE_ATTS = ['cx', 'cy', 'r', 'fill', 'stroke'];
const PATH_ATTS = ['d', 'fill', 'stroke'];
const RECT_ATTS = ['width', 'height', 'fill', 'stroke', 'x', 'y'];
const LINEARG_ATTS = ['id', 'x1', 'y1', 'x2', 'y2'];
const RADIALG_ATTS = ['id', 'cx', 'cy', 'r'];
const STOP_ATTS = ['offset'];
const ELLIPSE_ATTS = ['fill', 'cx', 'cy', 'rx', 'ry'];
const POLYGON_ATTS = ['points'];
let ind = 0;
class SvgUri extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.isComponentMounted = false;
        this.state = { svgXmlData: props.svgXmlData };
        this.createSVGElement = this.createSVGElement.bind(this);
        this.obtainComponentAttrs = this.obtainComponentAttrs.bind(this);
        this.inspectNode = this.inspectNode.bind(this);
        this.fecthSVGData = this.fecthSVGData.bind(this);
        // Gets the image data from an URL or a static file
        if (props.source) {
            const source = resolveAssetSource(props.source) || {};
            this.fecthSVGData(source.uri);
        }
    }
    UNSAFE_componentWillMount() {
        this.isComponentMounted = true;
    }
    UNSAFE_componentWillUnmount() {
        this.isComponentMounted = false;
    }
    UNSAFE_componentWillReceiveProps(nextProps) {
        if (nextProps.source) {
            const source = resolveAssetSource(nextProps.source) || {};
            const oldSource = resolveAssetSource(this.props.source) || {};
            if (source.uri !== oldSource.uri) {
                this.fecthSVGData(source.uri);
            }
        }
    }
    async fecthSVGData(uri) {
        let responseXML = null;
        try {
            let response = await fetch(uri);
            responseXML = await response.text();
        }
        catch (e) {
            console.error('ERROR SVG', e);
        }
        finally {
            if (this.isComponentMounted) {
                this.setState({ svgXmlData: responseXML || '' });
            }
        }
        return responseXML;
    }
    createSVGElement(node, child) {
        let componentAttrs = {};
        let i = ind++;
        switch (node.nodeName) {
            case 'svg':
                componentAttrs = this.obtainComponentAttrs(node, SVG_ATTS);
                if (this.props.width)
                    componentAttrs.width = this.props.width;
                if (this.props.height)
                    componentAttrs.height = this.props.height;
                return (<react_native_svg_1.default key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.default>);
            case 'g':
                componentAttrs = this.obtainComponentAttrs(node, G_ATTS);
                return (<react_native_svg_1.G key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.G>);
            case 'path':
                componentAttrs = this.obtainComponentAttrs(node, PATH_ATTS);
                return (<react_native_svg_1.Path key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Path>);
            case 'circle':
                componentAttrs = this.obtainComponentAttrs(node, CIRCLE_ATTS);
                return (<react_native_svg_1.Circle key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Circle>);
            case 'rect':
                componentAttrs = this.obtainComponentAttrs(node, RECT_ATTS);
                return (<react_native_svg_1.Rect key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Rect>);
            case 'linearGradient':
                componentAttrs = this.obtainComponentAttrs(node, LINEARG_ATTS);
                return (<react_native_svg_1.Defs key={i}>
            <react_native_svg_1.LinearGradient {...componentAttrs}>{child}</react_native_svg_1.LinearGradient>
          </react_native_svg_1.Defs>);
            case 'radialGradient':
                componentAttrs = this.obtainComponentAttrs(node, RADIALG_ATTS);
                return (<react_native_svg_1.Defs key={i}>
            <react_native_svg_1.RadialGradient {...componentAttrs}>{child}</react_native_svg_1.RadialGradient>
          </react_native_svg_1.Defs>);
            case 'stop':
                componentAttrs = this.obtainComponentAttrs(node, STOP_ATTS);
                return (<react_native_svg_1.Stop key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Stop>);
            case 'ellipse':
                componentAttrs = this.obtainComponentAttrs(node, ELLIPSE_ATTS);
                return (<react_native_svg_1.Ellipse key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Ellipse>);
            case 'polygon':
                componentAttrs = this.obtainComponentAttrs(node, POLYGON_ATTS);
                return (<react_native_svg_1.Polygon key={i} {...componentAttrs}>
            {child}
          </react_native_svg_1.Polygon>);
            default:
                return null;
        }
    }
    obtainComponentAttrs({ attributes }, enabledAttributes) {
        let styleAttrs = {};
        Array.from(attributes).forEach(({ nodeName, nodeValue }) => {
            Object.assign(styleAttrs, utils_1.transformStyle(nodeName, nodeValue, this.props.fill));
        });
        let componentAttrs = Array.from(attributes)
            // @ts-ignore
            .map(utils_1.camelCaseNodeName)
            .map(utils_1.removePixelsFromNodeValue)
            .filter(utils_1.getEnabledAttributes(enabledAttributes))
            .reduce((acc, { nodeName, nodeValue }) => ({
            ...acc,
            [nodeName]: this.props.fill && nodeName === 'fill' ? this.props.fill : nodeValue,
        }), {});
        Object.assign(componentAttrs, styleAttrs);
        return componentAttrs;
    }
    inspectNode(node) {
        //Process the xml node
        let arrayElements = [];
        // Only process accepted elements
        if (!ACCEPTED_SVG_ELEMENTS.includes(node.nodeName))
            return null;
        // if have children process them.
        // Recursive function.
        if (node.childNodes && node.childNodes.length > 0) {
            for (let i = 0; i < node.childNodes.length; i++) {
                let temp = this.inspectNode(node.childNodes[i]);
                if (temp != null)
                    arrayElements.push(temp);
            }
        }
        let element = this.createSVGElement(node, arrayElements);
        return element;
    }
    render() {
        try {
            if (this.state.svgXmlData == null)
                return null;
            let inputSVG = this.state.svgXmlData.substring(this.state.svgXmlData.indexOf('<svg '), this.state.svgXmlData.indexOf('</svg>') + 6);
            let doc = new xmldom_1.default.DOMParser().parseFromString(inputSVG);
            let rootSVG = this.inspectNode(doc.childNodes[0]);
            return <react_native_1.View style={this.props.style}>{rootSVG}</react_native_1.View>;
        }
        catch (e) {
            console.error('ERROR SVG', e);
            return null;
        }
    }
}
exports.default = SvgUri;
